package org.firstinspires.ftc.teamcode.subsystem;

import static org.firstinspires.ftc.teamcode.PedroConstants.WRIST_SERVO;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.wristBackwardBucket;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.wristBucket;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.wristFloor;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.wristGrabBlueberrySkyhook;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.wristHangBlueberrySkyhook;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.wristInit;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.wristPickup;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.wristSpeciemen;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.wristtravel;

import androidx.annotation.NonNull;

import com.acmerobotics.dashboard.telemetry.TelemetryPacket;
import com.acmerobotics.roadrunner.Action;
import com.acmerobotics.roadrunner.ftc.Actions;
import com.qualcomm.robotcore.hardware.HardwareMap;
import com.qualcomm.robotcore.hardware.ServoImplEx;

public class WristSubsystem {

    public enum WristState {
        FLOOR, BUCKET, PICKUP, RUNG, SPECIMEN, GRAB_BLUEBERRY_SKYHOOK, HANG_BLUEBERRY_SKYHOOK, INIT, REVERSE_BUCKET, TRAVEL
    }

    public ServoImplEx wrist;
    public WristState state;

    public WristSubsystem(HardwareMap hardwareMap) {
        this.wrist = hardwareMap.get(ServoImplEx.class, WRIST_SERVO);
    }

    public class MoveToPosition implements Action {
        private double positionValue;
        private WristState positionState;

        public MoveToPosition(double positionValue, WristState positionState) {
            this.positionValue = positionValue;
            this.positionState = positionState;
        }

        @Override
        public boolean run(@NonNull TelemetryPacket telemetryPacket) {
            setState(positionState);
            wrist.setPosition(positionValue);
            telemetryPacket.put("Wrist State", positionState);
            return false;
        }
    }
    public void grabBlueberrySkyhook(){
        setState(WristState.GRAB_BLUEBERRY_SKYHOOK);
        wrist.setPosition(wristGrabBlueberrySkyhook);
    }


    public void hangBlueberrySkyhook(){
        setState(WristState.HANG_BLUEBERRY_SKYHOOK);
        wrist.setPosition(wristHangBlueberrySkyhook);
    }
    public void toFloorPosition() {
        setState(WristState.FLOOR);
        wrist.setPosition(wristFloor);
    }

    public void toFloorPositionTeleop() {
        setState(WristState.FLOOR);
        wrist.setPosition(.425);
    }

    public void toBucketPosition() {
        setState(WristState.BUCKET);
        wrist.setPosition(wristBucket);
    }

    public void toPickupPosition() {
        setState(WristState.PICKUP);
        wrist.setPosition(wristPickup);
    }
    public void toInitPosition(){
        setState(WristState.INIT);
        wrist.setPosition(wristInit);
    }
    public void InitAuto(){
        wrist.resetDeviceConfigurationForOpMode();
        toInitPosition();

    }

    public void setState(WristState wristState) {
        this.state = wristState;
    }

    public void switchState() {
        if (state == WristState.FLOOR) {
            toPickupPosition();
        } else if (state == WristState.PICKUP) {
            toFloorPosition();
        }
    }
    public void toReverseBucket(){
        wrist.setPosition(wristBackwardBucket);
        setState(WristState.REVERSE_BUCKET);
    }
    public void toTravelPosition(){
        wrist.setPosition(wristtravel);
        setState(WristState.TRAVEL);
    }

    public WristState getState() {
        return this.state;
    }

    public void initTeleOp() {
        wrist.resetDeviceConfigurationForOpMode();
        toPickupPosition();
    }

    public void setPosition(double position) {
        wrist.setPosition(position);
    }

    public double getPosition() {
        return wrist.getPosition();
    }

}
